#include "stm32f10x.h"
#include "led.h"
//----------------------------------------------------------
#define SYSCLOCK 72000000U
//----------------------------------------------------------
#define TIM_EnableIT_UPDATE(TIMx) SET_BIT(TIMx->DIER, TIM_DIER_UIE)
#define TIM_EnableCounter(TIMx) SET_BIT(TIMx->CR1, TIM_CR1_CEN)
#define TIM_DisableCounter(TIMx) CLEAR_BIT(TIMx->CR1, TIM_CR1_CEN)
#define I2C_OWNADDRESS1_7BIT             0x00004000U
#define I2C_MODE_I2C                     0x00000000U
#define SLAVE_OWN_ADDRESS                       0xA0
#define I2C_REQUEST_WRITE                       0x00
#define I2C_REQUEST_READ                        0x01
//----------------------------------------------------------
__IO uint32_t tmpreg;
__IO uint32_t SysTick_CNT = 0;
__IO uint8_t tim2_count = 0;
extern uint8_t R1,R2,R3,R4;
extern uint16_t num_gl;
uint8_t rd_value[20] = {0};
uint8_t wr_value[20] = {0x14,0x13,0x12,0x11,0x10,
                        0x0F,0x0E,0x0D,0x0C,0x0B,
                        0x0A,0x09,0x08,0x07,0x06,
                        0x05,0x04,0x03,0x02,0x01};
//----------------------------------------------------------
__forceinline void delay(__IO uint32_t tck)
{
  while(tck)
  {
    tck--;
  }  
}
//----------------------------------------------------------
void delay_ms(uint32_t ms)
{
  MODIFY_REG(SysTick->VAL,SysTick_VAL_CURRENT_Msk,SYSCLOCK / 1000 - 1);
  SysTick_CNT = ms;
  while(SysTick_CNT) {}
}
//-----------------------------------------------------------
void RCC_DeInit(void)
{
  SET_BIT(RCC->CR, RCC_CR_HSION);
  while(READ_BIT(RCC->CR, RCC_CR_HSIRDY == RESET)) {}
  MODIFY_REG(RCC->CR, RCC_CR_HSITRIM, 0x80U);
  CLEAR_REG(RCC->CFGR);
  while (READ_BIT(RCC->CFGR, RCC_CFGR_SWS) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_PLLON);
  while (READ_BIT(RCC->CR, RCC_CR_PLLRDY) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_HSEON | RCC_CR_CSSON);
  while (READ_BIT(RCC->CR, RCC_CR_HSERDY) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_HSEBYP);
  //Reset all CSR flags
  SET_BIT(RCC->CSR, RCC_CSR_RMVF);
}
//----------------------------------------------------------
void SetSysClockTo72(void)
{
  SET_BIT(RCC->CR, RCC_CR_HSEON);
  while(READ_BIT(RCC->CR, RCC_CR_HSERDY == RESET)) {}
  //Enable the Prefetch Buffer
  CLEAR_BIT(FLASH->ACR, FLASH_ACR_PRFTBE);
  SET_BIT(FLASH->ACR, FLASH_ACR_PRFTBE);
  MODIFY_REG(FLASH->ACR, FLASH_ACR_LATENCY, FLASH_ACR_LATENCY_2);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_HPRE, RCC_CFGR_HPRE_DIV1);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE2, RCC_CFGR_PPRE2_DIV1);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE1, RCC_CFGR_PPRE1_DIV2);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL,
             RCC_CFGR_PLLSRC | RCC_CFGR_PLLMULL9);
  SET_BIT(RCC->CR, RCC_CR_PLLON);
  while(READ_BIT(RCC->CR, RCC_CR_PLLRDY) != (RCC_CR_PLLRDY)) {}
  MODIFY_REG(RCC->CFGR, RCC_CFGR_SW, RCC_CFGR_SW_PLL);
  while(READ_BIT(RCC->CFGR, RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL) {}
}
//----------------------------------------------------------
void SysTick_Init(void)
{
  MODIFY_REG(SysTick->LOAD,SysTick_LOAD_RELOAD_Msk,SYSCLOCK / 1000 - 1);
  CLEAR_BIT(SysTick->VAL, SysTick_VAL_CURRENT_Msk);
  SET_BIT(SysTick->CTRL, SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk | SysTick_CTRL_TICKINT_Msk);
}
//----------------------------------------------------------
static void TIM2_Init(void)
{
  SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM2EN);
  NVIC_EnableIRQ(TIM2_IRQn);
  WRITE_REG(TIM2->PSC, 3599);
  WRITE_REG(TIM2->ARR, 100);
}
//----------------------------------------------------------
static void GPIO_Init(void)
{
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPAEN |RCC_APB2ENR_IOPBEN);
  MODIFY_REG(GPIOA->CRL, GPIO_CRL_CNF7_0 | GPIO_CRL_CNF6_0 | GPIO_CRL_CNF5_0 | GPIO_CRL_CNF4_0,\
             GPIO_CRL_MODE7_0 | GPIO_CRL_MODE6_0 | GPIO_CRL_MODE5_0 | GPIO_CRL_MODE4_0);
  MODIFY_REG(GPIOB->CRH, GPIO_CRH_CNF15_0 | GPIO_CRH_CNF14_0 | GPIO_CRH_CNF13_0 | GPIO_CRH_CNF12_0 | \
             GPIO_CRH_CNF11_0 | GPIO_CRH_CNF10_0 | GPIO_CRH_CNF9_0 | GPIO_CRH_CNF8_0,\
             GPIO_CRH_MODE15_0 | GPIO_CRH_MODE14_0 | GPIO_CRH_MODE13_0 | GPIO_CRH_MODE12_0 | \
             GPIO_CRH_MODE11_0 | GPIO_CRH_MODE10_0 | GPIO_CRH_MODE9_0 | GPIO_CRH_MODE8_0);
  SET_BIT(GPIOB->ODR, GPIO_ODR_ODR15 | GPIO_ODR_ODR14 | GPIO_ODR_ODR13 | GPIO_ODR_ODR12 |\
             GPIO_ODR_ODR11 | GPIO_ODR_ODR10 | GPIO_ODR_ODR9 | GPIO_ODR_ODR8);
}
//----------------------------------------------------------
static void I2C_Init(void)
{
  //I2C1 GPIO
  SET_BIT(GPIOB->CRL, GPIO_CRL_CNF7_1 | GPIO_CRL_CNF6_1 | GPIO_CRL_CNF7_0 | GPIO_CRL_CNF6_0 |\
                  GPIO_CRL_MODE7_1 | GPIO_CRL_MODE6_1 | GPIO_CRL_MODE7_0 | GPIO_CRL_MODE6_0);
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPBEN);
  //RCC peripheral clock enabling
  SET_BIT(RCC->APB1ENR, RCC_APB1ENR_I2C1EN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB1ENR, RCC_APB1ENR_I2C1EN);
  //Disable  acknowledge on Own Address2 match address
  CLEAR_BIT(I2C1->OAR2, I2C_OAR2_ENDUAL);
  //Disable General Call
  CLEAR_BIT(I2C1->CR1, I2C_CR1_ENGC);
  //Enable Clock stretching
  CLEAR_BIT(I2C1->CR1, I2C_CR1_NOSTRETCH);
  //Disable I2C peripheral
  CLEAR_BIT(I2C1->CR1, I2C_CR1_PE);
  //ClockSpeed
  MODIFY_REG(I2C1->CR2, I2C_CR2_FREQ, 36);
  //MODIFY_REG(I2C1->TRISE, I2C_TRISE_TRISE, 36 + 1); //100
  MODIFY_REG(I2C1->TRISE, I2C_TRISE_TRISE, 11); //400
  //MODIFY_REG(I2C1->CCR, (I2C_CCR_FS | I2C_CCR_DUTY | I2C_CCR_CCR), 180); //100
  MODIFY_REG(I2C1->CCR, (I2C_CCR_FS | I2C_CCR_DUTY | I2C_CCR_CCR), 0x0000801E); //400
  //Set the Own Address1
  MODIFY_REG(I2C1->OAR1, I2C_OAR1_ADD0 | I2C_OAR1_ADD1_7 | I2C_OAR1_ADD8_9 | I2C_OAR1_ADDMODE, I2C_OWNADDRESS1_7BIT);
  //Configure I2C1 peripheral mode with parameter : I2C
  MODIFY_REG(I2C1->CR1, I2C_CR1_SMBUS | I2C_CR1_SMBTYPE | I2C_CR1_ENARP, I2C_MODE_I2C);
  //Enable I2C peripheral
  SET_BIT(I2C1->CR1, I2C_CR1_PE);
  //TypeAcknowledge
  MODIFY_REG(I2C1->CR1, I2C_CR1_ACK, I2C_CR1_ACK);
  //Set the 7bits Own Address2
  MODIFY_REG(I2C1->OAR2, I2C_OAR2_ADD2, 0);
}
//----------------------------------------------------------
void AT24C_WriteBytes (uint16_t addr,uint8_t *buf, uint16_t bytes_count)
{
  uint16_t i;
  //Disable Pos
  CLEAR_BIT(I2C1->CR1, I2C_CR1_POS);
  //Prepare the generation of a ACKnowledge condition
  //after the address receive match code or next received byte
  MODIFY_REG(I2C1->CR1, I2C_CR1_ACK, I2C_CR1_ACK);
  //Generate a START condition
  SET_BIT(I2C1->CR1, I2C_CR1_START);
  //Indicate the status of Start Bit (master mode)
  while (!READ_BIT(I2C1->SR1, I2C_SR1_SB)){};
  //read state
  (void) I2C1->SR1;
  //Transmit Address SLAVE
  MODIFY_REG(I2C1->DR, I2C_DR_DR, SLAVE_OWN_ADDRESS | I2C_REQUEST_WRITE);
  while (!READ_BIT(I2C1->SR1, I2C_SR1_ADDR)){};
  (void) I2C1->SR1;
  (void) I2C1->SR2;
  //Transmit Address begin EEPROM
  MODIFY_REG(I2C1->DR, I2C_DR_DR, (uint8_t) (addr>>8));
  while (!READ_BIT(I2C1->SR1, I2C_SR1_TXE)){};
  MODIFY_REG(I2C1->DR, I2C_DR_DR, (uint8_t) addr);
  while (!READ_BIT(I2C1->SR1, I2C_SR1_TXE)){};
  for(i=0;i<bytes_count;i++)
  {
    MODIFY_REG(I2C1->DR, I2C_DR_DR, buf[i]);
    while (!READ_BIT(I2C1->SR1, I2C_SR1_TXE)){};
  }
  SET_BIT(I2C1->CR1, I2C_CR1_STOP);
}
//----------------------------------------------------------
void AT24C_ReadBytes (uint16_t addr, uint8_t *buf, uint16_t bytes_count)
{
  uint16_t i;
  //Disable Pos
  CLEAR_BIT(I2C1->CR1, I2C_CR1_POS);
  //Prepare the generation of a ACKnowledge condition
  //after the address receive match code or next received byte
  MODIFY_REG(I2C1->CR1, I2C_CR1_ACK, I2C_CR1_ACK);
  //Generate a START condition
  SET_BIT(I2C1->CR1, I2C_CR1_START);
  //Indicate the status of Start Bit (master mode)
  while (!READ_BIT(I2C1->SR1, I2C_SR1_SB)){};
  //read state
  (void) I2C1->SR1;
  //Transmit Address
  MODIFY_REG(I2C1->DR, I2C_DR_DR, SLAVE_OWN_ADDRESS | I2C_REQUEST_WRITE);
  while (!READ_BIT(I2C1->SR1, I2C_SR1_ADDR)){};
  (void) I2C1->SR1;
  (void) I2C1->SR2;
  //Transmit Address begin EEPROM
  MODIFY_REG(I2C1->DR, I2C_DR_DR, (uint8_t) (addr>>8));
  while (!READ_BIT(I2C1->SR1, I2C_SR1_TXE)){};
  MODIFY_REG(I2C1->DR, I2C_DR_DR, (uint8_t) addr);
  while (!READ_BIT(I2C1->SR1, I2C_SR1_TXE)){};
  SET_BIT(I2C1->CR1, I2C_CR1_START);
  while (!READ_BIT(I2C1->SR1, I2C_SR1_SB)){};
  (void) I2C1->SR1;
  //Transmit Address    
  MODIFY_REG(I2C1->DR, I2C_DR_DR, SLAVE_OWN_ADDRESS | I2C_REQUEST_READ);
  while (!READ_BIT(I2C1->SR1, I2C_SR1_ADDR)){};
  (void) I2C1->SR1;
  (void) I2C1->SR2;
  for(i=0;i<bytes_count;i++)
  {
    if(i<(bytes_count-1))
    {
      while (!READ_BIT(I2C1->SR1, I2C_SR1_RXNE)){};
      buf[i] = READ_BIT(I2C1->DR, I2C_DR_DR);
    }
    else
    {
      CLEAR_BIT(I2C1->CR1, I2C_CR1_ACK);
      SET_BIT(I2C1->CR1, I2C_CR1_STOP);
      while (!READ_BIT(I2C1->SR1, I2C_SR1_RXNE)){};
      buf[i] = READ_BIT(I2C1->DR, I2C_DR_DR);
    }
  }
}
//----------------------------------------------------------
int main(void)
{
  uint16_t i;
  RCC_DeInit();
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_AFIOEN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB2ENR, RCC_APB2ENR_AFIOEN);
  //NOJTAG: JTAG-DP Disabled and SW-DP Enabled 
  CLEAR_BIT(AFIO->MAPR,AFIO_MAPR_SWJ_CFG);
  SET_BIT(AFIO->MAPR, AFIO_MAPR_SWJ_CFG_JTAGDISABLE);
  SetSysClockTo72();
  SysTick_Init();
  TIM2_Init();
  GPIO_Init();
  I2C_Init();
  TIM_EnableIT_UPDATE(TIM2);
  TIM_EnableCounter(TIM2);
  AT24C_WriteBytes (0x008A, wr_value, 20);
  //AT24C_ReadBytes (0x008A, rd_value, 20);
  for(i=0;i<20;i++)
  {
    ledprint(wr_value[i]);
    //ledprint(rd_value[i]);
    delay_ms(1000);
  }
  while(1)
	{
  }
}
//----------------------------------------------------------
void SysTick_Handler(void)
{
  if(SysTick_CNT > 0)  SysTick_CNT--;
}
//----------------------------------------------------------
void TIM2_IRQHandler(void)
{
  if(READ_BIT(TIM2->SR, TIM_SR_UIF))
  {
    CLEAR_BIT(TIM2->SR, TIM_SR_UIF);
    if(tim2_count==0)
    {
      MODIFY_REG(GPIOA->ODR, GPIO_ODR_ODR5 | GPIO_ODR_ODR6 | GPIO_ODR_ODR7, GPIO_ODR_ODR4);
      segchar(R1);
    }
    if(tim2_count==1)
    {
      MODIFY_REG(GPIOA->ODR, GPIO_ODR_ODR4 | GPIO_ODR_ODR6 | GPIO_ODR_ODR7, GPIO_ODR_ODR5);
      segchar(R2);
      if(num_gl<10)
        SET_BIT(GPIOB->ODR, GPIO_ODR_ODR15 | GPIO_ODR_ODR14 | GPIO_ODR_ODR13 | GPIO_ODR_ODR12 |\
                GPIO_ODR_ODR11 | GPIO_ODR_ODR10 | GPIO_ODR_ODR9 | GPIO_ODR_ODR8);
    }
    if(tim2_count==2)
    {
      MODIFY_REG(GPIOA->ODR, GPIO_ODR_ODR4 | GPIO_ODR_ODR5 | GPIO_ODR_ODR7, GPIO_ODR_ODR6);
      segchar(R3);
      if(num_gl<100)
        SET_BIT(GPIOB->ODR, GPIO_ODR_ODR15 | GPIO_ODR_ODR14 | GPIO_ODR_ODR13 | GPIO_ODR_ODR12 |\
                GPIO_ODR_ODR11 | GPIO_ODR_ODR10 | GPIO_ODR_ODR9 | GPIO_ODR_ODR8);
    }
    if(tim2_count==3)
    {
      MODIFY_REG(GPIOA->ODR, GPIO_ODR_ODR4 | GPIO_ODR_ODR5 | GPIO_ODR_ODR6, GPIO_ODR_ODR7);
      segchar(R4);
      if(num_gl<1000)
        SET_BIT(GPIOB->ODR, GPIO_ODR_ODR15 | GPIO_ODR_ODR14 | GPIO_ODR_ODR13 | GPIO_ODR_ODR12 |\
                GPIO_ODR_ODR11 | GPIO_ODR_ODR10 | GPIO_ODR_ODR9 | GPIO_ODR_ODR8);
    }
    tim2_count++;
    if(tim2_count>3) tim2_count=0;
  }
}
//----------------------------------------------------------
